// 
// Copyright (C) 2006, NinjaTrader LLC <www.ninjatrader.com>.
// NinjaTrader reserves the right to modify or overwrite this NinjaScript component with each release.
//

#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
	/// <summary>
	/// Daily Session Pivot Points.
	/// </summary>
	[Description("CMI PivotsDaily V5.")]
	public class CMIPivotsDaily_V5 : Indicator
	{
		#region Variables
		private	SolidBrush[]		brushes					= { new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black), 
															new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black), 
															new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black),  
															new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black),  
															new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black),
															new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black),
															new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black),
															new SolidBrush(Color.Black)};
		private DateTime			currentDate				= Cbi.Globals.MinDate;
		private DateTime			currentMonth			= Cbi.Globals.MinDate;
		private DateTime			currentWeek				= Cbi.Globals.MinDate;
		private DateTime  			marketTime				= Cbi.Globals.MinDate;
		private	double				currentOpen				= 0;
		private	double				currentHigh				= double.MinValue;
		private	double				currentLow				= double.MaxValue;
		private	double				currentClose			= 0;
		private Bars				dailyBars				= null;
		private	string				errorData				= "Insufficient historical data to calculate pivots. Increase chart look back period (days back).";
        private float				errorTextWidth			= 0;
        private float				errorTextHeight			= 0;
		private bool				existsHistDailyData		= false;
		private bool				isInitializing			= false;
		private bool				isDailyDataLoaded		= false;
		private Data.PivotRange		pivotRangeType			= PivotRange.Daily;
		private bool				firstPeriod				= true;
		private bool				setOHLC					= false;
		private HLCCalculationMode	priorDayHLC				= HLCCalculationMode.CalcFromIntradayData;
		private	double				pp						= 0;
		private	double				r1						= 0;
		private	double				r2						= 0;
		private	double				r3						= 0;
		private	double				r4						= 0;
		private	double				s1						= 0;
		private	double				s2						= 0;
		private	double				s3						= 0;
		private	double				s4						= 0;
		private	double				rmid					= 0;
		private	double				r12mid					= 0;
		private	double				r23mid					= 0;
		private	double				r34mid					= 0;
		private	double				smid					= 0;
		private	double				s12mid					= 0;
		private	double				s23mid					= 0;
		private	double				s34mid					= 0;
		private double				midline					= 0;
		private double				previousOpen			= 0;
		private double				previousHigh			= 0;
		private double				previousLow				= 0;
		private double				previousClose			= 0;
		private	StringFormat		stringFormatFar			= new StringFormat();
		private	StringFormat		stringFormatNear		= new StringFormat();
		private SolidBrush			textBrush				= new SolidBrush(Color.Black);
		private Font				textFont				= new Font("Arial", 30);
		private double				userDefinedOpen			= 0;
		private double				userDefinedHigh			= 0;
		private double				userDefinedLow			= 0;
		private double				userDefinedClose		= 0;
		private bool				useUserDefinedValues	= false;
		private int					width					= 30;
		private bool				instrumentfound			= true;
		private bool				showMidpivots			= true;
		private TimeSpan			sessionBreak			= new TimeSpan(0,0,0);
		private TimeSpan			sessionStart			= new TimeSpan(0,0,0);
		private TimeSpan			sessionLength			= new TimeSpan(0,0,0);
		private bool				chicago					= false;
		private bool				newyork					= false;
		private bool				london					= false;
		private bool				frankfurt				= false;
		private TimeSpan			daySpan					= new TimeSpan(24,0,0);
		private TimeSpan 			breakFOREX				= new TimeSpan(22,0,0);
		private TimeSpan			breakCMEIndex			= new TimeSpan(15,30,0);
		private TimeSpan			breakCMEInterest		= new TimeSpan(17,0,0);
		private TimeSpan			breakCMEForex			= new TimeSpan(16,0,0); 
		private TimeSpan			breakCMEAgri			= new TimeSpan(17,0,0);
		private TimeSpan			breakCMEMetals			= new TimeSpan(18,0,0);
		private TimeSpan			breakNYMEX				= new TimeSpan(18,0,0);
		private TimeSpan			breakNYBOT				= new TimeSpan(24,0,0);
		private TimeSpan			breakNYBOTRussell		= new TimeSpan(19,0,0);
		private TimeSpan			breakIPE				= new TimeSpan(24,0,0);
		private TimeSpan			breakEUREX				= new TimeSpan(24,0,0);
		private TimeSpan 			beginFOREX				= new TimeSpan(22,0,0);
		private TimeSpan			beginCMEIndex			= new TimeSpan(15,30,0);
		private TimeSpan			beginCMEInterest		= new TimeSpan(17,0,0);
		private TimeSpan			beginCMEForex			= new TimeSpan(16,0,0); 
		private TimeSpan			beginCMEAgri			= new TimeSpan(17,0,0);
		private TimeSpan			beginCMEMetals			= new TimeSpan(18,0,0);
		private TimeSpan			beginNYMEX				= new TimeSpan(18,0,0);
		private TimeSpan			beginNYBOT				= new TimeSpan(24,0,0);
		private TimeSpan			beginNYBOTRussell		= new TimeSpan(19,0,0);
		private TimeSpan			beginIPE				= new TimeSpan(24,0,0);
		private TimeSpan			beginEUREX				= new TimeSpan(24,0,0);
		private int					lastYear				= 2011;
		private DateTime[]			beginDST				= new DateTime[100];
		private	DateTime[]			endDST					= new DateTime[100];			

		#endregion

		/// <summary>
		/// This method is used to configure the indicator and is called once before any bar data is loaded.
		/// </summary>
		protected override void Initialize()
		{
			Add(new Plot(new Pen(Color.Aqua,2), PlotStyle.Line,"Daily PP "));
			Add(new Plot(new Pen(Color.Aqua,2), PlotStyle.Line,"Daily R1 "));
			Add(new Plot(new Pen(Color.Aqua,2), PlotStyle.Line,"Daily S1 "));
			Add(new Plot(new Pen(Color.Aqua,2), PlotStyle.Line,"Daily R2 "));
			Add(new Plot(new Pen(Color.Aqua,2), PlotStyle.Line,"Daily S2 "));
			Add(new Plot(new Pen(Color.Aqua,2), PlotStyle.Line,"Daily R3 "));
			Add(new Plot(new Pen(Color.Aqua,2), PlotStyle.Line,"Daily S3 "));
			Add(new Plot(new Pen(Color.Aqua,2), PlotStyle.Line,"Daily R4 "));
			Add(new Plot(new Pen(Color.Aqua,2), PlotStyle.Line,"Daily S4 "));
			
			Add(new Plot(new Pen(Color.Aqua,2), PlotStyle.Line,"Daily RMid "));
			Add(new Plot(new Pen(Color.Aqua,2), PlotStyle.Line,"Daily SMid "));
			Add(new Plot(new Pen(Color.Aqua,2), PlotStyle.Line,"Daily R12Mid "));
			Add(new Plot(new Pen(Color.Aqua,2), PlotStyle.Line,"Daily S12Mid "));
			Add(new Plot(new Pen(Color.Aqua,2), PlotStyle.Line,"Daily R23Mid "));
			Add(new Plot(new Pen(Color.Aqua,2), PlotStyle.Line,"Daily S23Mid "));
			Add(new Plot(new Pen(Color.Aqua,2), PlotStyle.Line,"Daily R34Mid "));
			Add(new Plot(new Pen(Color.Aqua,2), PlotStyle.Line,"Daily S34Mid "));
			
			Add(new Plot(new Pen(Color.Transparent,2), PlotStyle.Line,"PriorDay Open "));
			Add(new Plot(new Pen(Color.Lime,2), PlotStyle.Line,"PriorDay High "));
			Add(new Plot(new Pen(Color.OrangeRed,2), PlotStyle.Line,"PriorDay Low "));
			Add(new Plot(new Pen(Color.Gold,2), PlotStyle.Line,"PriorDay Close "));
			Add(new Plot(new Pen(Color.MediumSpringGreen,2), PlotStyle.Line,"PriorDay Midline "));

			CalculateOnBarClose			= false;     
			AutoScale					= false;
			Overlay						= true;
			stringFormatFar.Alignment	= StringAlignment.Far;
			
			// Defining all instruments for which this indicator can be used. Indicator will not show on any instruments not listed here!
			
			if(Instrument == null)
				return;
			if (Instrument.MasterInstrument.InstrumentType == Cbi.InstrumentType.Currency)
			{
				sessionBreak = breakFOREX;
				sessionStart = beginFOREX;
				london = true;
			}	
			if (Instrument.MasterInstrument.Name == "ES"||Instrument.MasterInstrument.Name == "EMD"||Instrument.MasterInstrument.Name == "YM"
				||Instrument.MasterInstrument.Name == "NQ" ||Instrument.MasterInstrument.Name == "SP500"||Instrument.MasterInstrument.Name == "NDX"
				||Instrument.MasterInstrument.Name == "RUT" ||Instrument.MasterInstrument.Name == "DJIA"||Instrument.MasterInstrument.Name == "NYA")
			{
				sessionBreak = breakCMEIndex;
				sessionStart = beginCMEIndex;
				chicago = true;
			}
			if (Instrument.MasterInstrument.Name == "ZB"||Instrument.MasterInstrument.Name == "ZN"||Instrument.MasterInstrument.Name == "ZF"
				||Instrument.MasterInstrument.Name == "ZT")
			{
				sessionBreak = breakCMEInterest;
				sessionStart = beginCMEInterest;
				chicago = true;
			}
			if (Instrument.MasterInstrument.Name == "6A"||Instrument.MasterInstrument.Name == "6B"||Instrument.MasterInstrument.Name == "6C"
				||Instrument.MasterInstrument.Name == "6E"||Instrument.MasterInstrument.Name == "6J"||Instrument.MasterInstrument.Name == "6S")
			{
				sessionBreak = breakCMEForex;
				sessionStart = beginCMEForex;
				chicago = true;
			}
			if (Instrument.MasterInstrument.Name == "ZC"||Instrument.MasterInstrument.Name == "ZL"||Instrument.MasterInstrument.Name == "ZM"
				||Instrument.MasterInstrument.Name == "ZS"||Instrument.MasterInstrument.Name == "ZW")
			{
				sessionBreak = breakCMEAgri;
				sessionStart = beginCMEAgri;
				chicago = true;
			}
			if (Instrument.MasterInstrument.Name == "ZG"||Instrument.MasterInstrument.Name == "ZI")
			{
				sessionBreak = breakCMEMetals;
				sessionStart = beginCMEMetals;
				chicago = true;
			}
			if (Instrument.MasterInstrument.Name == "CL"||Instrument.MasterInstrument.Name == "HO"||Instrument.MasterInstrument.Name == "RB"
				||Instrument.MasterInstrument.Name == "NG"||Instrument.MasterInstrument.Name == "GC"||Instrument.MasterInstrument.Name == "SI"
				||Instrument.MasterInstrument.Name == "HG"||Instrument.MasterInstrument.Name == "QM")
			{
				sessionBreak = breakNYMEX;
				sessionStart = beginNYMEX;
				newyork = true;
			}
			if (Instrument.MasterInstrument.Name == "CC"||Instrument.MasterInstrument.Name == "CT"||Instrument.MasterInstrument.Name == "KC"
				||Instrument.MasterInstrument.Name == "SB")
			{
				sessionBreak = breakNYBOT;
				sessionStart = beginNYBOT;
				newyork = true;
			}
			if (Instrument.MasterInstrument.Name == "TF")
			{
				sessionBreak = breakNYBOTRussell;
				sessionStart = beginNYBOTRussell;
				newyork = true;
			}
			if (Instrument.MasterInstrument.Name == "COIL"||Instrument.MasterInstrument.Name == "GOIL")
			{
				sessionBreak = breakIPE;
				sessionStart = beginIPE;
				london = true;
			}
			if (Instrument.MasterInstrument.Name == "FDAX"||Instrument.MasterInstrument.Name == "FESX"||Instrument.MasterInstrument.Name == "FGBL"
				||Instrument.MasterInstrument.Name == "FGBM")
			{
				sessionBreak = breakEUREX;
				sessionStart = beginEUREX;
				frankfurt = true;
			}
			if (sessionBreak > sessionStart) 
				sessionLength = sessionBreak-sessionStart;
			else if (sessionBreak < sessionStart)
				sessionLength = sessionBreak-sessionStart+daySpan;
			else
				sessionLength = daySpan;
			
			/// Calculating Daylight Saving Time
			
			if (newyork)
				for (int year = 2000; year <= lastYear; year++)
				{	
					if (year<=2007) beginDST[year-2000] = new DateTime(year,4,7-(5*year/4+4)%7,7,0,0);
					if (year<=2006) endDST[year-2000] = new DateTime(year,10,31-(5*year/4+1)%7,7,0,0);
					if (year>2007) beginDST[year-2000] = new DateTime(year,3,14-(5*year/4+1)%7,7,0,0);
					if (year>2006) endDST[year-2000] = new DateTime(year,11,7-(5*year/4+1)%7,7,0,0);
			 	}
			else if (chicago)
				for (int year = 2000; year <= lastYear; year++)
				{	
					if (year<=2007) beginDST[year-2000] = new DateTime(year,4,7-(5*year/4+4)%7,8,0,0);
					if (year<=2006) endDST[year-2000] = new DateTime(year,10,31-(5*year/4+1)%7,8,0,0);
					if (year>2007) beginDST[year-2000] = new DateTime(year,3,14-(5*year/4+1)%7,8,0,0);
					if (year>2006) endDST[year-2000] = new DateTime(year,11,7-(5*year/4+1)%7,7,8,0);
			 	}
				else if (london)
				for (int year = 2000; year <= lastYear; year++)
				{	
					beginDST[year-2000] = new DateTime(year,3,31-(5*year/4+4)%7,1,0,0);
					endDST[year-2000] = new DateTime(year,10,31-(5*year/4+1)%7,1,0,0);
				}
			else if (frankfurt)
				for (int year = 2000; year <= lastYear; year++)
				{	
					beginDST[year-2000] = new DateTime(year,3,31-(5*year/4+4)%7,1,0,0);
					endDST[year-2000] = new DateTime(year,10,31-(5*year/4+1)%7,1,0,0);
				}
			else 
				{
					instrumentfound = false;
				}	
		}

		/// <summary>
		/// Called on each bar update event (incoming tick)
		/// </summary>
		protected override void OnBarUpdate()
		{
			if (Bars == null)
				return; 
			if (!Data.BarsType.GetInstance(Bars.Period.Id).IsIntraday && Bars.Period.Id != PeriodType.Day)
				return;
			if (Bars.Period.Id == PeriodType.Day && pivotRangeType == PivotRange.Daily)
				return;
			if (Bars.Period.Id == PeriodType.Day && Bars.Period.Value > 1)
				return;
			if (priorDayHLC == HLCCalculationMode.CalcFromIntradayData && (Bars.Period.Id != PeriodType.Minute || instrumentfound == false))
				return;
			
			if (!isDailyDataLoaded && !isInitializing)
			{
				isInitializing = true;
				if (priorDayHLC == HLCCalculationMode.DailyBars) 
				{
					dailyBars		= Data.Bars.GetBars(Bars.Instrument, new Period(PeriodType.Day, 1), Bars.From, Bars.To, (Session) Bars.Session.Clone(), Data.Bars.SplitAdjust, Data.Bars.DividendAdjust);
					existsHistDailyData	= (dailyBars.Count <= 1) ? false : true;
					firstPeriod = false;
				}
				else 
					existsHistDailyData = false;
				
				isInitializing 	  = false;
				isDailyDataLoaded = true;
			}

			IBar	dailyBar;
			if (existsHistDailyData) 
			{
				DateTime intradayBarTime = Time[0].Date;
				if (Data.BarsType.GetInstance(Bars.Period.Id).IsIntraday)
				{				
					intradayBarTime = ConvertTime(Time[0]);
				}
				if (intradayBarTime.DayOfWeek != DayOfWeek.Saturday) 
					dailyBar = dailyBars.Get(dailyBars.GetBar(intradayBarTime.Date));
				else
					dailyBar = dailyBars.Get(dailyBars.GetBar((intradayBarTime.AddDays(-1)).Date));
			}
			else
				dailyBar = null;
		
			double	open		= existsHistDailyData ? dailyBar.Open : Open[0];
			double	high		= existsHistDailyData ? dailyBar.High : High[0];
			double	low			= existsHistDailyData ? dailyBar.Low : Low[0];
			double	close		= existsHistDailyData ? dailyBar.Close : Close[0];
			
			marketTime = Time[0];
			if (Data.BarsType.GetInstance(Bars.Period.Id).IsIntraday)
			{
				marketTime = ConvertTime(marketTime);
				if (pivotRangeType == PivotRange.Monthly && RoundUpTimeToPeriodTime(marketTime.Date, PivotRange.Monthly).DayOfWeek == DayOfWeek.Sunday)
				marketTime = marketTime.AddDays(2);
			}
			
			if ((currentDate != Cbi.Globals.MinDate && pivotRangeType == PivotRange.Daily && marketTime.Date != currentDate && currentDate.DayOfWeek != DayOfWeek.Saturday && currentDate.DayOfWeek != DayOfWeek.Sunday)
				|| (currentWeek != Cbi.Globals.MinDate && pivotRangeType == PivotRange.Weekly && RoundUpTimeToPeriodTime(marketTime.Date, PivotRange.Weekly) != currentWeek) 
				|| (currentMonth != Cbi.Globals.MinDate && pivotRangeType == PivotRange.Monthly && RoundUpTimeToPeriodTime(marketTime.Date, PivotRange.Monthly) != currentMonth))
			{
				if (firstPeriod == false)
				{
						pp				= (currentHigh + currentLow + currentClose) / 3;
						s1				= 2 * pp - currentHigh;
						r1				= 2 * pp - currentLow;
						s2				= pp - (currentHigh - currentLow);
						r2				= pp + (currentHigh - currentLow);
						s3				= pp - 2 * (currentHigh - currentLow);
						r3				= pp + 2 * (currentHigh - currentLow);
						s4				= pp - 3 * (currentHigh - currentLow);
						r4				= pp + 3 * (currentHigh - currentLow);
						smid			= 0.5*(pp+s1);
						rmid			= 0.5*(pp+r1);
						s12mid			= 0.5*(s1+s2);
						r12mid			= 0.5*(r1+r2);
						s23mid			= 0.5*(s2+s3);
						r23mid			= 0.5*(r2+r3);
						s34mid			= 0.5*(s3+s4);
						r34mid			= 0.5*(r3+r4);
						previousOpen	= currentOpen;
						previousHigh	= currentHigh;
						previousLow		= currentLow;
						previousClose	= currentClose;
						midline			= (currentHigh + currentLow) / 2;
				}
				firstPeriod = false;
				setOHLC = false;
				if (marketTime.TimeOfDay >= daySpan-sessionLength)
				{	
					currentOpen		= (priorDayHLC == HLCCalculationMode.UserDefinedValues) ? userDefinedOpen : open;
					currentHigh		= (priorDayHLC == HLCCalculationMode.UserDefinedValues) ? userDefinedHigh : high;
					currentLow		= (priorDayHLC == HLCCalculationMode.UserDefinedValues) ? userDefinedLow : low;
					currentClose	= (priorDayHLC == HLCCalculationMode.UserDefinedValues) ? userDefinedClose : close;
					setOHLC = true;
				}
			}
			else if (marketTime.TimeOfDay >= daySpan-sessionLength)
			{
				currentClose	= (priorDayHLC == HLCCalculationMode.UserDefinedValues) ? userDefinedClose : close;
				if (setOHLC == false)
				{	
				currentOpen		= (priorDayHLC == HLCCalculationMode.UserDefinedValues) ? userDefinedOpen : open;
				currentHigh		= (priorDayHLC == HLCCalculationMode.UserDefinedValues) ? userDefinedHigh : high;
				currentLow		= (priorDayHLC == HLCCalculationMode.UserDefinedValues) ? userDefinedLow : low;
				setOHLC			=  true;	
				}
				else	
				{
				currentHigh		= (priorDayHLC == HLCCalculationMode.UserDefinedValues) ? userDefinedHigh : Math.Max(currentHigh, high);
				currentLow		= (priorDayHLC == HLCCalculationMode.UserDefinedValues) ? userDefinedLow : Math.Min(currentLow, low);
				}
			}

			if (pivotRangeType == PivotRange.Daily)
				currentDate = marketTime.Date;
			if (pivotRangeType == PivotRange.Weekly)
				currentWeek = RoundUpTimeToPeriodTime(marketTime.Date, PivotRange.Weekly);
			if (pivotRangeType == PivotRange.Monthly)
				currentMonth = RoundUpTimeToPeriodTime(marketTime.Date, PivotRange.Monthly);

			if ((pivotRangeType == PivotRange.Daily && currentDate != Cbi.Globals.MinDate)
				|| (pivotRangeType == PivotRange.Weekly && currentWeek != Cbi.Globals.MinDate)
				|| (pivotRangeType == PivotRange.Monthly && currentMonth != Cbi.Globals.MinDate))
			{
				PP.Set(pp);
				R1.Set(r1);
				S1.Set(s1);
				R2.Set(r2);
				S2.Set(s2);
				R3.Set(r3);
				S3.Set(s3);
				R4.Set(r4);
				S4.Set(s4);

				if (showMidpivots == true)
				{
					RMid.Set(rmid);
					SMid.Set(smid);
					R12Mid.Set(r12mid);
					S12Mid.Set(s12mid);
					R23Mid.Set(r23mid);
					S23Mid.Set(s23mid);
					R34Mid.Set(r34mid);
					S34Mid.Set(s34mid);
				}	
				PreviousOpen.Set(previousOpen);
				PreviousHigh.Set(previousHigh);
				PreviousLow.Set(previousLow);
				PreviousClose.Set(previousClose);
				Midline.Set(midline);
			}
		}

		#region Properties
		/// <summary>
		/// </summary>
		///[Description("Type of period for pivot points.")]
		///[Category("Parameters")]
		///public Data.PivotRange PivotRangeType 
		///{
		///	get { return pivotRangeType; }
		///	set { pivotRangeType = value; }
		///}

		/// <summary>
		/// </summary>
		[Description("Approach for calculation the prior day HLC values.")]
		[Category("Parameters")]
		public Data.HLCCalculationMode PriorDayHLC
		{
			get { return priorDayHLC; }
			set { priorDayHLC = value; }
		}
		
		/// <summary>
		/// </summary>
		[Description("Option to show MidPivots")]
		[Category("Parameters")]
		public bool ShowMidpivots 
		{
			get { return showMidpivots; }
			set { showMidpivots = value; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries PP
		{
			get { return Values[0]; }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries R1
		{
			get { return Values[1]; }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries R2
		{
			get { return Values[3]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries R3
		{
			get { return Values[5]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries R4
		{
			get { return Values[7]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries S1
		{
			get { return Values[2]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries S2
		{
			get { return Values[4]; }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries S3
		{
			get { return Values[6]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries S4
		{
			get { return Values[8]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries RMid
		{
			get { return Values[9]; }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries R12Mid
		{
			get { return Values[11]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries R23Mid
		{
			get { return Values[13]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries R34Mid
		{
			get { return Values[15]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries SMid
		{
			get { return Values[10]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries S12Mid
		{
			get { return Values[12]; }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries S23Mid
		{
			get { return Values[14]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries S34Mid
		{
			get { return Values[16]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries PreviousOpen
		{
			get { return Values[17]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries PreviousHigh
		{
			get { return Values[18]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries PreviousLow
		{
			get { return Values[19]; }
		}
			
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries PreviousClose
		{
			get { return Values[20]; }
		}
			
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries Midline
		{
			get { return Values[21]; }
		}
			
		/// <summary>
		/// close value for user defined pivots calculation
		/// </summary>
		[Description("User defined prior session open value used for pivots calculation.")]
		[Category("\rUser defined values")]
		public double UserDefinedOpen
		{
			get { return userDefinedOpen; }
			set { userDefinedOpen = value; }
		}

		/// <summary>
		/// high value for user defined pivots calculation
		/// </summary>
		[Description("User defined prior session high value used for pivots calculation.")]
		[Category("\rUser defined values")]
		public double UserDefinedHigh
		{
			get { return userDefinedHigh; }
			set { userDefinedHigh = value; }
		}

		/// <summary>
		/// low value for user defined pivots calculation
		/// </summary>
		[Description("User defined prior session low value used for pivots calculation.")]
		[Category("\rUser defined values")]
		public double UserDefinedLow
		{
			get { return userDefinedLow; }
			set { userDefinedLow = value; }
		}

		/// <summary>
		/// close value for user defined pivots calculation
		/// </summary>
		[Description("User defined prior session close value used for pivots calculation.")]
		[Category("\rUser defined values")]
		public double UserDefinedClose
		{
			get { return userDefinedClose; }
			set { userDefinedClose = value; }
		}
		
		/// <summary>
		/// </summary>
		[Description("Width of the pivot lines as # of bars.")]
		[Category("Parameters")]
		public int Width
		{
			get { return width; }
			set { width = Math.Max(1, value); }
		}
				
		///<summary
		///</summary>
		[XmlIgnore]
		[Browsable(false)]
		public TimeSpan BreakFOREX
		{
			get { return breakFOREX;}
			set { breakFOREX = value;}
		}	
	
		///<summary
		///</summary>
		[Description("Session Closing Time for FOREX markets \nPlease Use GMT!")]
		[Category("User defined values")]
		public string S_End_FOREX_GMT	
		{
			get 
			{ 
				return string.Format("{0:D2}:{1:D2}", breakFOREX.Hours, breakFOREX.Minutes);
			}
			set 
			{ 
			string[]values =((string)value).Split(':');
			breakFOREX = new TimeSpan(Convert.ToInt16(values[0]),Convert.ToInt16(values[1]),0);
			}
		}
		
		///<summary
		///</summary>
		[XmlIgnore]
		[Browsable(false)]
		public TimeSpan BreakCMEIndex
		{
			get { return breakCMEIndex;}
			set { breakCMEIndex = value;}
		}	
	
		///<summary
		///</summary>
		[Description("Session Closing Time for Globex Index Futures \nPlease Use Central Time!")]
		[Category("User defined values")]
		public string S_End_CME_Index	
		{
			get 
			{ 
				return string.Format("{0:D2}:{1:D2}", breakCMEIndex.Hours, breakCMEIndex.Minutes);
			}
			set 
			{ 
			string[]values =((string)value).Split(':');
			breakCMEIndex = new TimeSpan(Convert.ToInt16(values[0]),Convert.ToInt16(values[1]),0);
			}
		}
				
		///<summary
		///</summary>
		[XmlIgnore]
		[Browsable(false)]
		public TimeSpan BreakCMEInterest
		{
			get { return breakCMEInterest;}
			set { breakCMEInterest = value;}
		}	
	
		///<summary
		///</summary>
		[Description("Session Closing Time for Globex Interest Futures \nPlease Use Central Time!")]
		[Category("User defined values")]
		public string S_End_CME_Interest	
		{
			get 
			{ 
				return string.Format("{0:D2}:{1:D2}", breakCMEInterest.Hours, breakCMEInterest.Minutes);
			}
			set 
			{ 
			string[]values =((string)value).Split(':');
			breakCMEInterest = new TimeSpan(Convert.ToInt16(values[0]),Convert.ToInt16(values[1]),0);
			}
		}
				
		///<summary
		///</summary>
		[XmlIgnore]
		[Browsable(false)]
		public TimeSpan BreakCMEForex
		{
			get { return breakCMEForex;}
			set { breakCMEForex = value;}
		}	
	
		///<summary
		///</summary>
		[Description("Session Closing Time for Globex Currency Futures \nPlease Use Central Time!")]
		[Category("User defined values")]
		public string S_End_CME_Forex	
		{
			get 
			{ 
				return string.Format("{0:D2}:{1:D2}", breakCMEForex.Hours, breakCMEForex.Minutes);
			}
			set 
			{ 
			string[]values =((string)value).Split(':');
			breakCMEForex = new TimeSpan(Convert.ToInt16(values[0]),Convert.ToInt16(values[1]),0);
			}
		}
				
		///<summary
		///</summary>
		[XmlIgnore]
		[Browsable(false)]
		public TimeSpan BreakCMEAgri
		{
			get { return breakCMEAgri;}
			set { breakCMEAgri = value;}
		}	
	
		///<summary
		///</summary>
		[Description("Session Closing Time for Globex Agricultural Futures \nPlease Use Central Time!")]
		[Category("User defined values")]
		public string S_End_CME_Agricultural	
		{
			get 
			{ 
				return string.Format("{0:D2}:{1:D2}", breakCMEAgri.Hours, breakCMEAgri.Minutes);
			}
			set 
			{ 
			string[]values =((string)value).Split(':');
			breakCMEAgri = new TimeSpan(Convert.ToInt16(values[0]),Convert.ToInt16(values[1]),0);
			}
		}

		///<summary
		///</summary>
		[XmlIgnore]
		[Browsable(false)]
		public TimeSpan BreakCMEMetals
		{
			get { return breakCMEMetals;}
			set { breakCMEMetals = value;}
		}	
	
		///<summary
		///</summary>
		[Description("Session Closing Time for Globex Metals Futures \nPlease Use Central Time!")]
		[Category("User defined values")]
		public string S_End_CME_Metals	
		{
			get 
			{ 
				return string.Format("{0:D2}:{1:D2}", breakCMEMetals.Hours, breakCMEMetals.Minutes);
			}
			set 
			{ 
			string[]values =((string)value).Split(':');
			breakCMEMetals = new TimeSpan(Convert.ToInt16(values[0]),Convert.ToInt16(values[1]),0);
			}
		}
		
		///<summary
		///</summary>
		[XmlIgnore]
		[Browsable(false)]
		public TimeSpan BreakNYMEX
		{
			get { return breakNYMEX;}
			set { breakNYMEX = value;}
		}	
	
		///<summary
		///</summary>
		[Description("Session Closing Time for NYMEX Futures \nPlease Use Eastern Time!")]
		[Category("User defined values")]
		public string S_End_NYMEX	
		{
			get 
			{ 
				return string.Format("{0:D2}:{1:D2}", breakNYMEX.Hours, breakNYMEX.Minutes);
			}
			set 
			{ 
			string[]values =((string)value).Split(':');
			breakNYMEX = new TimeSpan(Convert.ToInt16(values[0]),Convert.ToInt16(values[1]),0);
			}
		}
		
		///<summary
		///</summary>
		[XmlIgnore]
		[Browsable(false)]
		public TimeSpan BreakNYBOT
		{
			get { return breakNYBOT;}
			set { breakNYBOT = value;}
		}	
	
		///<summary
		///</summary>
		[Description("Session Closing Time for NYBOT Futures \nPlease Use Eastern Time!")]
		[Category("User defined values")]
		public string S_End_NYBOT	
		{
			get 
			{ 
				return string.Format("{0:D2}:{1:D2}", breakNYBOT.Hours, breakNYBOT.Minutes);
			}
			set 
			{ 
			string[]values =((string)value).Split(':');
			breakNYBOT = new TimeSpan(Convert.ToInt16(values[0]),Convert.ToInt16(values[1]),0);
			}
		}
		
		///<summary
		///</summary>
		[XmlIgnore]
		[Browsable(false)]
		public TimeSpan BreakNYBOTRussell
		{
			get { return breakNYBOTRussell;}
			set { breakNYBOTRussell = value;}
		}	
	
		///<summary
		///</summary>
		[Description("Session Closing Time for NYBOT Russell Futures \nPlease Use Eastern Time!")]
		[Category("User defined values")]
		public string S_End_NYBOT_Russell	
		{
			get 
			{ 
				return string.Format("{0:D2}:{1:D2}", breakNYBOTRussell.Hours, breakNYBOTRussell.Minutes);
			}
			set 
			{ 
			string[]values =((string)value).Split(':');
			breakNYBOTRussell = new TimeSpan(Convert.ToInt16(values[0]),Convert.ToInt16(values[1]),0);
			}
		}
		
		///<summary
		///</summary>
		[XmlIgnore]
		[Browsable(false)]
		public TimeSpan BreakIPE
		{
			get { return breakIPE;}
			set { breakIPE = value;}
		}	
	
		///<summary
		///</summary>
		[Description("Session Closing Time for IPE Futures \nPlease Use GMT!")]
		[Category("User defined values")]
		public string S_End_IPE	
		{
			get 
			{ 
				return string.Format("{0:D2}:{1:D2}", breakIPE.Hours, breakIPE.Minutes);
			}
			set 
			{ 
			string[]values =((string)value).Split(':');
			breakIPE = new TimeSpan(Convert.ToInt16(values[0]),Convert.ToInt16(values[1]),0);
			}
		}
		
		///<summary
		///</summary>
		[XmlIgnore]
		[Browsable(false)]
		public TimeSpan BreakEUREX
		{
			get { return breakEUREX;}
			set { breakEUREX = value;}
		}	
	
		///<summary
		///</summary>
		[Description("Session Closing Time for EUREX Futures \nPlease Use Central European Time!")]
		[Category("User defined values")]
		public string S_End_EUREX	
		{
			get 
			{ 
				return string.Format("{0:D2}:{1:D2}", breakEUREX.Hours, breakEUREX.Minutes);
			}
			set 
			{ 
			string[]values =((string)value).Split(':');
			breakEUREX = new TimeSpan(Convert.ToInt16(values[0]),Convert.ToInt16(values[1]),0);
			}
		}
		
		///<summary
		///</summary>
		[Description("Session Opening Time for FOREX markets \nPlease Use GMT!")]
		[Category("User defined values")]
		public string S_Begin_FOREX_GMT	
		{
			get 
			{ 
				return string.Format("{0:D2}:{1:D2}", beginFOREX.Hours, beginFOREX.Minutes);
			}
			set 
			{ 
			string[]values =((string)value).Split(':');
			beginFOREX = new TimeSpan(Convert.ToInt16(values[0]),Convert.ToInt16(values[1]),0);
			}
		}
		
		///<summary
		///</summary>
		[XmlIgnore]
		[Browsable(false)]
		public TimeSpan BeginCMEIndex
		{
			get { return beginCMEIndex;}
			set { beginCMEIndex = value;}
		}	
	
		///<summary
		///</summary>
		[Description("Session Opening Time for Globex Index Futures \nPlease Use Central Time!")]
		[Category("User defined values")]
		public string S_Begin_CME_Index	
		{
			get 
			{ 
				return string.Format("{0:D2}:{1:D2}", beginCMEIndex.Hours, beginCMEIndex.Minutes);
			}
			set 
			{ 
			string[]values =((string)value).Split(':');
			beginCMEIndex = new TimeSpan(Convert.ToInt16(values[0]),Convert.ToInt16(values[1]),0);
			}
		}
				
		///<summary
		///</summary>
		[XmlIgnore]
		[Browsable(false)]
		public TimeSpan BeginCMEInterest
		{
			get { return beginCMEInterest;}
			set { beginCMEInterest = value;}
		}	
	
		///<summary
		///</summary>
		[Description("Session Opening Time for Globex Interest Futures \nPlease Use Central Time!")]
		[Category("User defined values")]
		public string S_Begin_CME_Interest	
		{
			get 
			{ 
				return string.Format("{0:D2}:{1:D2}", beginCMEInterest.Hours, beginCMEInterest.Minutes);
			}
			set 
			{ 
			string[]values =((string)value).Split(':');
			beginCMEInterest = new TimeSpan(Convert.ToInt16(values[0]),Convert.ToInt16(values[1]),0);
			}
		}
				
		///<summary
		///</summary>
		[XmlIgnore]
		[Browsable(false)]
		public TimeSpan BeginCMEForex
		{
			get { return beginCMEForex;}
			set { beginCMEForex = value;}
		}	
	
		///<summary
		///</summary>
		[Description("Session Opening Time for Globex Currency Futures \nPlease Use Central Time!")]
		[Category("User defined values")]
		public string S_Begin_CME_Forex	
		{
			get 
			{ 
				return string.Format("{0:D2}:{1:D2}", beginCMEForex.Hours, beginCMEForex.Minutes);
			}
			set 
			{ 
			string[]values =((string)value).Split(':');
			beginCMEForex = new TimeSpan(Convert.ToInt16(values[0]),Convert.ToInt16(values[1]),0);
			}
		}
				
		///<summary
		///</summary>
		[XmlIgnore]
		[Browsable(false)]
		public TimeSpan BeginCMEAgri
		{
			get { return beginCMEAgri;}
			set { beginCMEAgri = value;}
		}	
	
		///<summary
		///</summary>
		[Description("Session Opening Time for Globex Agricultural Futures \nPlease Use Central Time!")]
		[Category("User defined values")]
		public string S_Begin_CME_Agricultural	
		{
			get 
			{ 
				return string.Format("{0:D2}:{1:D2}", beginCMEAgri.Hours, beginCMEAgri.Minutes);
			}
			set 
			{ 
			string[]values =((string)value).Split(':');
			beginCMEAgri = new TimeSpan(Convert.ToInt16(values[0]),Convert.ToInt16(values[1]),0);
			}
		}

		///<summary
		///</summary>
		[XmlIgnore]
		[Browsable(false)]
		public TimeSpan BeginCMEMetals
		{
			get { return beginCMEMetals;}
			set { beginCMEMetals = value;}
		}	
	
		///<summary
		///</summary>
		[Description("Session Opening Time for Globex Metals Futures \nPlease Use Central Time!")]
		[Category("User defined values")]
		public string S_Begin_CME_Metals	
		{
			get 
			{ 
				return string.Format("{0:D2}:{1:D2}", beginCMEMetals.Hours, beginCMEMetals.Minutes);
			}
			set 
			{ 
			string[]values =((string)value).Split(':');
			beginCMEMetals = new TimeSpan(Convert.ToInt16(values[0]),Convert.ToInt16(values[1]),0);
			}
		}
		
		///<summary
		///</summary>
		[XmlIgnore]
		[Browsable(false)]
		public TimeSpan BeginNYMEX
		{
			get { return beginNYMEX;}
			set { beginNYMEX = value;}
		}	
	
		///<summary
		///</summary>
		[Description("Session Opening Time for NYMEX Futures \nPlease Use Eastern Time!")]
		[Category("User defined values")]
		public string S_Begin_NYMEX	
		{
			get 
			{ 
				return string.Format("{0:D2}:{1:D2}", beginNYMEX.Hours, beginNYMEX.Minutes);
			}
			set 
			{ 
			string[]values =((string)value).Split(':');
			beginNYMEX = new TimeSpan(Convert.ToInt16(values[0]),Convert.ToInt16(values[1]),0);
			}
		}
		
		///<summary
		///</summary>
		[XmlIgnore]
		[Browsable(false)]
		public TimeSpan BeginNYBOT
		{
			get { return beginNYBOT;}
			set { beginNYBOT = value;}
		}	
	
		///<summary
		///</summary>
		[Description("Session Opening Time for NYBOT Futures \nPlease Use Eastern Time!")]
		[Category("User defined values")]
		public string S_Begin_NYBOT	
		{
			get 
			{ 
				return string.Format("{0:D2}:{1:D2}", beginNYBOT.Hours, beginNYBOT.Minutes);
			}
			set 
			{ 
			string[]values =((string)value).Split(':');
			beginNYBOT = new TimeSpan(Convert.ToInt16(values[0]),Convert.ToInt16(values[1]),0);
			}
		}
			
		///<summary
		///</summary>
		[XmlIgnore]
		[Browsable(false)]
		public TimeSpan BeginNYBOTRussell
		{
			get { return beginNYBOTRussell;}
			set { beginNYBOTRussell = value;}
		}	
	
		///<summary
		///</summary>
		[Description("Session Opening Time for NYBOT Russell Futures \nPlease Use Eastern Time!")]
		[Category("User defined values")]
		public string S_Begin_NYBOT_Russell	
		{
			get 
			{ 
				return string.Format("{0:D2}:{1:D2}", beginNYBOTRussell.Hours, beginNYBOTRussell.Minutes);
			}
			set 
			{ 
			string[]values =((string)value).Split(':');
			beginNYBOTRussell = new TimeSpan(Convert.ToInt16(values[0]),Convert.ToInt16(values[1]),0);
			}
		}
		
		///<summary
		///</summary>
		[XmlIgnore]
		[Browsable(false)]
		public TimeSpan BeginIPE
		{
			get { return beginIPE;}
			set { beginIPE = value;}
		}	
	
		///<summary
		///</summary>
		[Description("Session Opening Time for IPE Futures \nPlease Use GMT!")]
		[Category("User defined values")]
		public string S_Begin_IPE	
		{
			get 
			{ 
				return string.Format("{0:D2}:{1:D2}", beginIPE.Hours, beginIPE.Minutes);
			}
			set 
			{ 
			string[]values =((string)value).Split(':');
			beginIPE = new TimeSpan(Convert.ToInt16(values[0]),Convert.ToInt16(values[1]),0);
			}
		}
		
		///<summary
		///</summary>
		[XmlIgnore]
		[Browsable(false)]
		public TimeSpan BeginEUREX
		{
			get { return beginEUREX;}
			set { beginEUREX = value;}
		}	
	
		///<summary
		///</summary>
		[Description("Session Opening Time for EUREX Futures \nPlease Use Central European Time!")]
		[Category("User defined values")]
		public string S_Begin_EUREX	
		{
			get 
			{ 
				return string.Format("{0:D2}:{1:D2}", beginEUREX.Hours, beginEUREX.Minutes);
			}
			set 
			{ 
			string[]values =((string)value).Split(':');
			beginEUREX = new TimeSpan(Convert.ToInt16(values[0]),Convert.ToInt16(values[1]),0);
			}
		}
		#endregion

		
		#region Miscellaneous

		private DateTime RoundUpTimeToPeriodTime(DateTime time, PivotRange pivotRange)
			
			/// simplified
		{
			if (pivotRange == PivotRange.Weekly)
			{
				return  time.AddDays(6 - Convert.ToInt16(time.DayOfWeek)).Date;
			}
			else if (pivotRange == PivotRange.Monthly)
			{
				DateTime result = new DateTime(time.Year, time.Month, 1); 
				return result.AddMonths(1).AddDays(-1);
			}
			else
				return time;
		}
		
		/// <summary>
		/// </summary>
		/// <param name="graphics"></param>
		/// <param name="bounds"></param>
		/// <param name="min"></param>
		/// <param name="max"></param>
		public override void Plot(Graphics graphics, Rectangle bounds, double min, double max)
		{
			if (Bars == null || ChartControl == null)
				return;

			// plot error if data not complete
			if (textBrush.Color != ChartControl.AxisColor || textFont != ChartControl.Font)
			{
				textBrush.Color = ChartControl.AxisColor;
                textFont = ChartControl.Font;

				SizeF errorSize	= graphics.MeasureString(errorData, textFont);
				errorTextWidth	= errorSize.Width + 5;
				errorTextHeight	= errorSize.Height + 5;
			}
			
			if (priorDayHLC == HLCCalculationMode.CalcFromIntradayData) 
			{
				DateTime firstBarTime = Bars.Get(0).Time;
				DateTime lastBarTime = Bars.Get(Math.Min(Bars.Count - 1, ChartControl.LastBarPainted)).Time;
				if 	(Data.BarsType.GetInstance(Bars.Period.Id).IsIntraday)
				{
					firstBarTime = ConvertTime(firstBarTime);
					lastBarTime = ConvertTime(lastBarTime);
				}
				if ((pivotRangeType == PivotRange.Monthly && Bars.Count > 0 && RoundUpTimeToPeriodTime(firstBarTime.Date, PivotRange.Monthly).AddMonths(1) >= lastBarTime.Date)
					|| (pivotRangeType == PivotRange.Weekly && Bars.Count > 0 && RoundUpTimeToPeriodTime(firstBarTime.Date, PivotRange.Weekly).AddDays(7) >= lastBarTime.Date)
				    || (pivotRangeType == PivotRange.Daily && Bars.Count > 0 && ((firstBarTime.Date.DayOfWeek == DayOfWeek.Friday)? firstBarTime.Date.AddDays(3): firstBarTime.Date.AddDays(1)) >= lastBarTime.Date))
				{
					graphics.DrawString(errorData, ChartControl.Font, textBrush, bounds.X + bounds.Width - errorTextWidth, bounds.Y + bounds.Height - errorTextHeight, stringFormatNear);
				}
			}
			else if (priorDayHLC == HLCCalculationMode.DailyBars && existsHistDailyData)
			{
				DateTime	firstBarTime = dailyBars.Get(0).Time;
				DateTime	lastBarTime = dailyBars.Get(Math.Min(dailyBars.Count - 1, ChartControl.LastBarPainted)).Time;
				if ((pivotRangeType == PivotRange.Monthly && Bars.Count > 0 && RoundUpTimeToPeriodTime(firstBarTime.Date, PivotRange.Monthly).AddMonths(1) >= lastBarTime.Date)
					|| (pivotRangeType == PivotRange.Weekly && Bars.Count > 0 && RoundUpTimeToPeriodTime(firstBarTime.Date, PivotRange.Weekly).AddDays(7) >= lastBarTime.Date)
					|| (pivotRangeType == PivotRange.Daily && Bars.Count > 0 && firstBarTime.Date.AddDays(1) >= lastBarTime.Date))
				{
					graphics.DrawString(errorData, ChartControl.Font, textBrush, bounds.X + bounds.Width - errorTextWidth, bounds.Y + bounds.Height - errorTextHeight, stringFormatNear);
				}
			}

			int	barWidth = ChartControl.ChartStyle.GetBarPaintWidth(ChartControl.BarWidth);
			for (int seriesCount = 0; seriesCount < Values.Length; seriesCount++)
			{
				SolidBrush		brush				= brushes[seriesCount];
				DateTime		lastDate			= Cbi.Globals.MinDate;
				DateTime		lastWeek			= Cbi.Globals.MinDate;
				DateTime		lastMonth			= Cbi.Globals.MinDate;
				int				lastX				= -1;
				int				lastY				= -1;
				SmoothingMode	oldSmoothingMode	= graphics.SmoothingMode;
				GraphicsPath	path				= new GraphicsPath();
				Gui.Chart.Plot	plot				= Plots[seriesCount];
				DataSeries	series					= (DataSeries) Values[seriesCount];

				if (brush.Color != plot.Pen.Color)	
					brush = new SolidBrush(plot.Pen.Color);

				for (int count = ChartControl.BarsPainted - 1; count >= ChartControl.BarsPainted - Math.Min(ChartControl.BarsPainted, Width); count--)
				{
					int whiteBars = Math.Max(0, ChartControl.LastBarPainted - ChartControl.Bars[0].Count + 1);
					int idx = ChartControl.LastBarPainted - whiteBars - ChartControl.BarsPainted + 1 + count;
					if (idx - Displacement < 0 || idx - Displacement >= Bars.Count || (!ChartControl.ShowBarsRequired && idx - Displacement < BarsRequired))
						continue;
					else if (!series.IsValidPlot(idx))
						continue;
					
					DateTime barTime = Bars.Get(idx).Time;
					if 	(Data.BarsType.GetInstance(Bars.Period.Id).IsIntraday)
						barTime = ConvertTime(barTime);
					if (pivotRangeType == PivotRange.Daily && lastDate == Cbi.Globals.MinDate) 
						lastDate = barTime.Date;
					else if (pivotRangeType == PivotRange.Weekly && lastWeek == Cbi.Globals.MinDate) 
						lastWeek = RoundUpTimeToPeriodTime(barTime.Date, PivotRange.Weekly);
					else if (pivotRangeType == PivotRange.Monthly && lastMonth == Cbi.Globals.MinDate) 
						lastMonth = RoundUpTimeToPeriodTime(barTime.Date, PivotRange.Monthly);
					else if (pivotRangeType == PivotRange.Daily && lastDate != barTime.Date 
							|| pivotRangeType == PivotRange.Weekly && lastWeek != RoundUpTimeToPeriodTime(barTime.Date, PivotRange.Weekly)
							|| pivotRangeType == PivotRange.Monthly && lastMonth != RoundUpTimeToPeriodTime(barTime.Date, PivotRange.Monthly))
						continue;

					double		val = series.Get(idx);
					int			x	= (int) (ChartControl.CanvasRight - ChartControl.BarMarginRight - barWidth / 2
										- (ChartControl.BarsPainted - 1) * ChartControl.BarSpace + (count - whiteBars) * ChartControl.BarSpace);
					int			y	= (int) ((bounds.Y + bounds.Height) - ((val - min ) / Gui.Chart.ChartControl.MaxMinusMin(max, min)) * bounds.Height);

					if (lastX >= 0)
					{
						if (y != lastY) // Problem here is, that last bar of old day has date of new day
							y = lastY;
						path.AddLine(lastX - plot.Pen.Width / 2, lastY, x - plot.Pen.Width / 2, y);
					}
					lastX	= x;
					lastY	= y;
				}

				graphics.SmoothingMode = SmoothingMode.AntiAlias;
				graphics.DrawPath(plot.Pen, path);
				graphics.SmoothingMode = oldSmoothingMode;
				graphics.DrawString(plot.Name, ChartControl.Font, brush, lastX, lastY - ChartControl.Font.GetHeight() / 2, stringFormatFar);
			}
		}

		private DateTime ConvertTime(DateTime time)
		{
			TimeSpan		localTimeChicago		= new TimeSpan(-6,0,0);
			TimeSpan		localTimeNewYork		= new TimeSpan(-5,0,0);
			TimeSpan		localTimeLondon			= new TimeSpan(0,0,0);
			TimeSpan		localTimeFrankfurt		= new TimeSpan(1,0,0);
			DateTime 		inputTime 				= time;
			
			time = time.ToUniversalTime().AddSeconds(-1);
			if (time.Year > lastYear)
				return inputTime;
			
			if ( time > beginDST[time.Year-2000] && time < endDST[time.Year-2000])
					time = time + new TimeSpan (1,0,0);
			if (sessionBreak >= new TimeSpan(14,0,0) && sessionBreak <= new TimeSpan(24,0,0))
					time = time.AddDays(1);
			if (newyork)
					return time = time + localTimeNewYork - sessionBreak;
			else if (chicago)
					return time = time + localTimeChicago - sessionBreak;
			else if (london)
					return time = time + localTimeLondon - sessionBreak;
			else if (frankfurt)
					return time = time + localTimeFrankfurt - sessionBreak;
			else
				return inputTime;
		}	
		#endregion
	}
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private CMIPivotsDaily_V5[] cacheCMIPivotsDaily_V5 = null;

        private static CMIPivotsDaily_V5 checkCMIPivotsDaily_V5 = new CMIPivotsDaily_V5();

        /// <summary>
        /// CMI PivotsDaily V5.
        /// </summary>
        /// <returns></returns>
        public CMIPivotsDaily_V5 CMIPivotsDaily_V5(Data.HLCCalculationMode priorDayHLC, bool showMidpivots, int width)
        {
            return CMIPivotsDaily_V5(Input, priorDayHLC, showMidpivots, width);
        }

        /// <summary>
        /// CMI PivotsDaily V5.
        /// </summary>
        /// <returns></returns>
        public CMIPivotsDaily_V5 CMIPivotsDaily_V5(Data.IDataSeries input, Data.HLCCalculationMode priorDayHLC, bool showMidpivots, int width)
        {
            checkCMIPivotsDaily_V5.PriorDayHLC = priorDayHLC;
            priorDayHLC = checkCMIPivotsDaily_V5.PriorDayHLC;
            checkCMIPivotsDaily_V5.ShowMidpivots = showMidpivots;
            showMidpivots = checkCMIPivotsDaily_V5.ShowMidpivots;
            checkCMIPivotsDaily_V5.Width = width;
            width = checkCMIPivotsDaily_V5.Width;

            if (cacheCMIPivotsDaily_V5 != null)
                for (int idx = 0; idx < cacheCMIPivotsDaily_V5.Length; idx++)
                    if (cacheCMIPivotsDaily_V5[idx].PriorDayHLC == priorDayHLC && cacheCMIPivotsDaily_V5[idx].ShowMidpivots == showMidpivots && cacheCMIPivotsDaily_V5[idx].Width == width && cacheCMIPivotsDaily_V5[idx].EqualsInput(input))
                        return cacheCMIPivotsDaily_V5[idx];

            CMIPivotsDaily_V5 indicator = new CMIPivotsDaily_V5();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.PriorDayHLC = priorDayHLC;
            indicator.ShowMidpivots = showMidpivots;
            indicator.Width = width;
            indicator.SetUp();

            CMIPivotsDaily_V5[] tmp = new CMIPivotsDaily_V5[cacheCMIPivotsDaily_V5 == null ? 1 : cacheCMIPivotsDaily_V5.Length + 1];
            if (cacheCMIPivotsDaily_V5 != null)
                cacheCMIPivotsDaily_V5.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheCMIPivotsDaily_V5 = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// CMI PivotsDaily V5.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.CMIPivotsDaily_V5 CMIPivotsDaily_V5(Data.HLCCalculationMode priorDayHLC, bool showMidpivots, int width)
        {
            return _indicator.CMIPivotsDaily_V5(Input, priorDayHLC, showMidpivots, width);
        }

        /// <summary>
        /// CMI PivotsDaily V5.
        /// </summary>
        /// <returns></returns>
        public Indicator.CMIPivotsDaily_V5 CMIPivotsDaily_V5(Data.IDataSeries input, Data.HLCCalculationMode priorDayHLC, bool showMidpivots, int width)
        {
            return _indicator.CMIPivotsDaily_V5(input, priorDayHLC, showMidpivots, width);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// CMI PivotsDaily V5.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.CMIPivotsDaily_V5 CMIPivotsDaily_V5(Data.HLCCalculationMode priorDayHLC, bool showMidpivots, int width)
        {
            return _indicator.CMIPivotsDaily_V5(Input, priorDayHLC, showMidpivots, width);
        }

        /// <summary>
        /// CMI PivotsDaily V5.
        /// </summary>
        /// <returns></returns>
        public Indicator.CMIPivotsDaily_V5 CMIPivotsDaily_V5(Data.IDataSeries input, Data.HLCCalculationMode priorDayHLC, bool showMidpivots, int width)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.CMIPivotsDaily_V5(input, priorDayHLC, showMidpivots, width);
        }

    }
}
#endregion
